@@ -26,14 +26,12 @@ extern crate lazy_static;
 extern crate log;
 extern crate open;
 extern crate regex;
-extern crate rustc_serialize;
-extern crate semver;
+extern crate serde;
 extern crate shaman;
 extern crate toml;
 
-#[cfg(feature = "chan")]
 #[macro_use]
-extern crate chan;
+extern crate crossbeam_channel;
 
 /**
 If this is set to `true`, the digests used for package IDs will be replaced with "stub" to make testing a bit easier.  Obviously, you don't want this `true` for release...
@@ -45,24 +43,6 @@ If this is set to `false`, then code that automatically deletes stuff *won't*.
 */
 const ALLOW_AUTO_REMOVE: bool = true;
 
-/**
-Length of time to suppress Cargo output.
-*/
-#[cfg(feature = "suppress-cargo-output")]
-const CARGO_OUTPUT_TIMEOUT: u64 = 2_000/*ms*/;
-
-// This macro exists for 1.11 support.
-#[cfg(windows)]
-macro_rules! if_windows {
-    (@as_expr $e:expr) => { $e };
-    ($($tts:tt)*) => { if_windows! { @as_expr { $($tts)* } } };
-}
-
-#[cfg(not(windows))]
-macro_rules! if_windows {
-    ($($tts:tt)*) => {{}};
-}
-
 mod consts;
 mod error;
 mod manifest;
@@ -76,30 +56,19 @@ mod file_assoc;
 #[cfg(not(windows))]
 mod file_assoc {}
 
-use semver::Version;
-use std::borrow::Cow;
+use serde::{Deserialize, Serialize};
 use std::ffi::OsString;
 use std::fs;
 use std::io::{Read, Write};
 use std::path::{Path, PathBuf};
 use std::process::{self, Command};
 
-use error::{Blame, MainError, Result, ResultExt};
-use platform::MigrationKind;
-use util::{ChainMap, Defer, PathExt};
-
-#[derive(Debug)]
-enum SubCommand {
-    Script(Args),
-    Templates(templates::Args),
-    #[cfg(windows)]
-    FileAssoc(file_assoc::Args),
-}
+use crate::error::{Blame, MainError, Result};
+use crate::util::{ChainMap, Defer};
 
 #[derive(Debug)]
 struct Args {
-    script: Option<String>,
-    args: Vec<String>,
+    command: Vec<String>,
     features: Option<String>,
 
     expr: bool,
@@ -112,14 +81,13 @@ struct Args {
     clear_cache: bool,
     debug: bool,
     dep: Vec<String>,
-    dep_extern: Vec<String>,
     extern_: Vec<String>,
     force: bool,
     unstable_features: Vec<String>,
     use_bincache: Option<bool>,
-    migrate_data: Option<MigrationKind>,
     build_kind: BuildKind,
     template: Option<String>,
+    list_templates: bool,
 }
 
 #[derive(Copy, Clone, Debug)]
@@ -130,83 +98,66 @@ enum BuildKind {
 }
 
 impl BuildKind {
-    fn can_exec_directly(&self) -> bool {
+    const fn can_exec_directly(&self) -> bool {
         match *self {
-            BuildKind::Normal => true,
-            BuildKind::Test | BuildKind::Bench => false,
+            Self::Normal => true,
+            Self::Test | Self::Bench => false,
         }
     }
 
     fn exec_command(&self) -> &'static str {
         match *self {
-            BuildKind::Normal => panic!("asked for exec command for normal build"),
-            BuildKind::Test => "test",
-            BuildKind::Bench => "bench",
+            Self::Normal => panic!("asked for exec command for normal build"),
+            Self::Test => "test",
+            Self::Bench => "bench",
         }
     }
 
     fn from_flags(test: bool, bench: bool) -> Self {
         match (test, bench) {
-            (false, false) => BuildKind::Normal,
-            (true, false) => BuildKind::Test,
-            (false, true) => BuildKind::Bench,
+            (false, false) => Self::Normal,
+            (true, false) => Self::Test,
+            (false, true) => Self::Bench,
             _ => panic!("got both test and bench"),
         }
     }
 }
 
-fn parse_args() -> SubCommand {
-    use clap::{App, AppSettings, Arg, ArgGroup, SubCommand};
+fn parse_args() -> Args {
+    use clap::{App, Arg, ArgGroup};
     let version = option_env!("CARGO_PKG_VERSION").unwrap_or("unknown");
     let about = r#"Compiles and rtuns a Rust script."#;
 
-    // "const str array slice"
-    macro_rules! csas {
-        ($($es:expr),*) => {
-            {
-                const PIN: &'static [&'static str] = &[$($es),*];
-                PIN
-            }
-        }
-    }
-
     // We have to kinda lie about who we are for the output to look right...
     let m = App::new("rust-script")
         .bin_name("rust-script")
         .version(version)
+        .setting(clap::AppSettings::TrailingVarArg)
+        //.setting(clap::AppSettings::AllowLeadingHyphen)
         .about(about)
-            //.usage("cargo script [FLAGS OPTIONS] [--] <script> <args>...")
-
-            /*
-            Major script modes.
-            */
-            .arg(Arg::with_name("script")
-                .help("Script file (with or without extension) to execute.")
+            .arg(Arg::with_name("command")
+                .help("Command (script file plus arguments) to execute.")
                 .index(1)
-            )
-            .arg(Arg::with_name("args")
-                .help("Additional arguments passed to the script.")
-                .index(2)
                 .multiple(true)
             )
             .arg(Arg::with_name("expr")
                 .help("Execute <script> as a literal expression and display the result.")
                 .long("expr")
                 .short("e")
-                .conflicts_with_all(csas!["loop"])
-                .requires("script")
+                .takes_value(false)
+                //.conflicts_with_all(&["loop"])
+                .requires("command")
             )
             .arg(Arg::with_name("loop")
                 .help("Execute <script> as a literal closure once for each line from stdin.")
                 .long("loop")
                 .short("l")
-                .conflicts_with_all(csas!["expr"])
-                .requires("script")
+                //.conflicts_with_all(&["expr"])
+                .requires("command")
             )
             .group(ArgGroup::with_name("expr_or_loop")
                 .args(&["expr", "loop"])
             )
-
             /*
             Options that impact the script being executed.
             */
@@ -227,7 +178,7 @@ fn parse_args() -> SubCommand {
                 .takes_value(true)
                 .multiple(true)
                 .number_of_values(1)
-                .requires("script")
+                .requires("command")
             )
             .arg(Arg::with_name("dep_extern")
                 .help("Like `dep`, except that it *also* adds a `#[macro_use] extern crate name;` item for expression and loop scripts.  Note that this only works if the name of the dependency and the name of the library it generates are exactly the same.")
@@ -266,7 +217,7 @@ fn parse_args() -> SubCommand {
                 .help("Build the script, but don't run it.")
                 .long("build-only")
                 .requires("script")
-                .conflicts_with_all(csas!["args"])
+                .conflicts_with_all(&["args"])
             )
             .arg(Arg::with_name("clear_cache")
                 .help("Clears out the script cache.")
@@ -281,36 +232,30 @@ fn parse_args() -> SubCommand {
                 .help("Generate the Cargo package, but don't compile or run it.")
                 .long("gen-pkg-only")
                 .requires("script")
-                .conflicts_with_all(csas!["args", "build_only", "debug", "force", "test", "bench"])
+                .conflicts_with_all(&["args", "build_only", "debug", "force", "test", "bench"])
             )
             .arg(Arg::with_name("pkg_path")
                 .help("Specify where to place the generated Cargo package.")
                 .long("pkg-path")
                 .takes_value(true)
-                .requires("script")
-                .conflicts_with_all(csas!["clear_cache", "force"])
+                .requires("command")
+                .conflicts_with_all(&["clear_cache", "force"])
             )
             .arg(Arg::with_name("use_bincache")
                 .help("Override whether or not the shared binary cache will be used for compilation.")
                 .long("use-shared-binary-cache")
                 .takes_value(true)
-                .possible_values(csas!["no", "yes"])
-            )
-            .arg(Arg::with_name("migrate_data")
-                .help("Migrate data from older versions.")
-                .long("migrate-data")
-                .takes_value(true)
-                .possible_values(csas!["dry-run", "for-real"])
+                .possible_values(&["no", "yes"])
             )
             .arg(Arg::with_name("test")
                 .help("Compile and run tests.")
                 .long("test")
-                .conflicts_with_all(csas!["bench", "debug", "args", "force"])
+                .conflicts_with_all(&["bench", "debug", "args", "force"])
             )
             .arg(Arg::with_name("bench")
                 .help("Compile and run benchmarks.  Requires a nightly toolchain.")
                 .long("bench")
-                .conflicts_with_all(csas!["test", "debug", "args", "force"])
+                .conflicts_with_all(&["test", "debug", "args", "force"])
             )
             .arg(Arg::with_name("template")
                 .help("Specify a template to use for expression scripts.")
@@ -319,13 +264,20 @@ fn parse_args() -> SubCommand {
                 .takes_value(true)
                 .requires("expr")
             )
+        .arg(Arg::with_name("list-templates")
+            .help("List the available templates.")
+            .long("list-templates")
+            .takes_value(false)
+            // TODO: .conflicts_with()
+        )
         .get_matches();
 
     fn owned_vec_string<'a, I>(v: Option<I>) -> Vec<String>
     where
         I: ::std::iter::Iterator<Item = &'a str>,
     {
-        v.map(|itr| itr.map(Into::into).collect()).unwrap_or(vec![])
+        v.map(|itr| itr.map(Into::into).collect())
+            .unwrap_or_default()
     }
 
     fn yes_or_no(v: Option<&str>) -> Option<bool> {
@@ -336,17 +288,8 @@ fn parse_args() -> SubCommand {
         })
     }
 
-    fn run_kind(v: Option<&str>) -> Option<MigrationKind> {
-        v.map(|v| match v {
-            "dry-run" => MigrationKind::DryRun,
-            "for-real" => MigrationKind::ForReal,
-            _ => unreachable!(),
-        })
-    }
-
-    ::SubCommand::Script(Args {
-        script: m.value_of("script").map(Into::into),
-        args: owned_vec_string(m.values_of("args")),
+    Args {
+        command: owned_vec_string(m.values_of("command")),
         features: m.value_of("features").map(Into::into),
 
         expr: m.is_present("expr"),
@@ -359,19 +302,18 @@ fn parse_args() -> SubCommand {
         clear_cache: m.is_present("clear_cache"),
         debug: m.is_present("debug"),
         dep: owned_vec_string(m.values_of("dep")),
-        dep_extern: owned_vec_string(m.values_of("dep_extern")),
         extern_: owned_vec_string(m.values_of("extern")),
         force: m.is_present("force"),
         unstable_features: owned_vec_string(m.values_of("unstable_features")),
         use_bincache: yes_or_no(m.value_of("use_bincache")),
-        migrate_data: run_kind(m.value_of("migrate_data")),
         build_kind: BuildKind::from_flags(m.is_present("test"), m.is_present("bench")),
         template: m.value_of("template").map(Into::into),
-    })
+        list_templates: m.is_present("list-templates"),
+    }
 }
 
 fn main() {
-    env_logger::init().unwrap();
+    env_logger::init();
     info!("starting");
     info!("args: {:?}", std::env::args().collect::<Vec<_>>());
     let stderr = &mut std::io::stderr();
@@ -395,40 +337,7 @@ fn try_main() -> Result<i32> {
     let args = parse_args();
     info!("Arguments: {:?}", args);
 
-    let args = match args {
-        SubCommand::Script(args) => args,
-        SubCommand::Templates(args) => return templates::try_main(args),
-        #[cfg(windows)]
-        SubCommand::FileAssoc(args) => return file_assoc::try_main(args),
-    };
-
-    /*
-    Do data migration before anything else, since it can cause the location of stuff to change.
-    */
-    if let Some(run_kind) = args.migrate_data {
-        println!("Migrating data...");
-        let (log, res) = platform::migrate_old_data(run_kind);
-        match (log.len(), res) {
-            (0, Ok(())) => {
-                println!("Nothing to do.");
-                return Ok(0);
-            }
-            (_, Ok(())) => {
-                for entry in log {
-                    println!("- {}", entry);
-                }
-                return Ok(0);
-            }
-            (_, Err(err)) => {
-                for entry in log {
-                    println!("- {}", entry);
-                }
-                return Err(err);
-            }
-        }
-    }
-
-    if log_enabled!(log::LogLevel::Debug) {
+    if log_enabled!(log::Level::Debug) {
         let scp = get_script_cache_path()?;
         let bcp = get_binary_cache_path()?;
         debug!("script-cache path: {:?}", scp);
@@ -445,27 +354,32 @@ fn try_main() -> Result<i32> {
         clean_cache(0)?;
 
         // If we *did not* get a `<script>` argument, that's OK.
-        if args.script.is_none() {
+        if args.command.is_empty() {
             // Just let the user know that we did *actually* run.
             println!("cargo script cache cleared.");
             return Ok(0);
         }
     }
 
+    if args.list_templates {
+        templates::list()?;
+        return Ok(0);
+    }
+
     // Take the arguments and work out what our input is going to be.  Primarily, this gives us the content, a user-friendly name, and a cache-friendly ID.
     // These three are just storage for the borrows we'll actually use.
     let script_name: String;
     let script_path: PathBuf;
     let content: String;
 
-    let input = match (args.script, args.expr, args.loop_) {
+    let input = match (args.command.get(0).cloned(), args.expr, args.loop_) {
         (Some(script), false, false) => {
             let (path, mut file) = find_script(script).ok_or("could not find script")?;
 
             script_name = path
                 .file_stem()
                 .map(|os| os.to_string_lossy().into_owned())
-                .unwrap_or("unknown".into());
+                .unwrap_or_else(|| "unknown".into());
 
             let mut body = String::new();
             file.read_to_string(&mut body)?;
@@ -479,14 +393,14 @@ fn try_main() -> Result<i32> {
         }
         (Some(expr), true, false) => {
             content = expr;
-            Input::Expr(&content, args.template.as_ref().map(|s| &**s))
+            Input::Expr(&content, args.template.as_deref())
         }
         (Some(loop_), false, true) => {
             content = loop_;
             Input::Loop(&content, args.count)
         }
-        (None, _, _) => Err((Blame::Human, consts::NO_ARGS_MESSAGE))?,
-        _ => Err((Blame::Human, "cannot specify both --expr and --loop"))?,
+        (None, _, _) => return Err((Blame::Human, consts::NO_ARGS_MESSAGE).into()),
+        _ => return Err((Blame::Human, "cannot specify both --expr and --loop").into()),
     };
     info!("input: {:?}", input);
 
@@ -502,7 +416,7 @@ fn try_main() -> Result<i32> {
         use std::collections::HashMap;
 
         let mut deps: HashMap<String, String> = HashMap::new();
-        for dep in args.dep.iter().chain(args.dep_extern.iter()).cloned() {
+        for dep in args.dep.iter().cloned() {
             // Append '=*' if it needs it.
             let dep = match dep.find('=') {
                 Some(_) => dep,
@@ -518,11 +432,11 @@ fn try_main() -> Result<i32> {
             );
 
             if name == "" {
-                Err((Blame::Human, "cannot have empty dependency package name"))?;
+                return Err((Blame::Human, "cannot have empty dependency package name").into());
             }
 
             if version == "" {
-                Err((Blame::Human, "cannot have empty dependency version"))?;
+                return Err((Blame::Human, "cannot have empty dependency version").into());
             }
 
             match deps.entry(name.into()) {
@@ -532,14 +446,15 @@ fn try_main() -> Result<i32> {
                 Occupied(oe) => {
                     // This is *only* a problem if the versions don't match.  We won't try to do anything clever in terms of upgrading or resolving or anything... exact match or go home.
                     let existing = oe.get();
-                    if &version != existing {
-                        Err((
+                    if version != existing {
+                        return Err((
                             Blame::Human,
                             format!(
                                 "conflicting versions for dependency '{}': '{}', '{}'",
                                 name, existing, version
                             ),
-                        ))?;
+                        )
+                            .into());
                     }
                 }
             }
@@ -560,47 +475,20 @@ fn try_main() -> Result<i32> {
             .unstable_features
             .iter()
             .map(|uf| format!("#![feature({})]", uf));
-        let dep_externs = args
-            .dep_extern
-            .iter()
-            .map(|d| match d.find('=') {
-                Some(i) => &d[..i],
-                None => &d[..],
-            })
-            .map(|d| match d.contains('-') {
-                true => Cow::from(d.replace("-", "_")),
-                false => Cow::from(d),
-            })
-            .map(|d| format!("#[macro_use] extern crate {};", d));
 
         let externs = args
             .extern_
             .iter()
             .map(|n| format!("#[macro_use] extern crate {};", n));
 
-        let mut items: Vec<_> = unstable_features
-            .chain(dep_externs)
-            .chain(externs)
-            .collect();
+        let mut items: Vec<_> = unstable_features.chain(externs).collect();
         items.sort();
         items
     };
     info!("prelude_items: {:?}", prelude_items);
 
     // Work out what to do.
-    let action = decide_action_for(
-        &input,
-        deps,
-        prelude_items,
-        args.debug,
-        args.pkg_path,
-        args.gen_pkg_only,
-        args.build_only,
-        args.force,
-        args.features,
-        args.use_bincache,
-        args.build_kind,
-    )?;
+    let action = decide_action_for(&input, deps, prelude_items, &args)?;
     info!("action: {:?}", action);
 
     gen_pkg_and_compile(&input, &action)?;
@@ -625,7 +513,7 @@ fn try_main() -> Result<i32> {
         let add_env = hint(move |cmd| {
             cmd.env(
                 "CARGO_SCRIPT_SCRIPT_PATH",
-                input.path().unwrap_or(Path::new("")),
+                input.path().unwrap_or_else(|| Path::new("")),
             );
             cmd.env("CARGO_SCRIPT_SAFE_NAME", input.safe_name());
             cmd.env("CARGO_SCRIPT_PKG_NAME", input.package_name());
@@ -638,7 +526,7 @@ fn try_main() -> Result<i32> {
             info!("executing {:?}", exe_path);
             match {
                 Command::new(exe_path)
-                    .args(&args.args)
+                    .args(&args.command[1..])
                     .chain_map(add_env)
                     .status()
                     .map(|st| st.code().unwrap_or(1))
@@ -667,7 +555,7 @@ Clean up the cache folder.
 
 Looks for all folders whose metadata says they were created at least `max_age` in the past and kills them dead.
 */
-fn clean_cache(max_age: u64) -> Result<()> {
+fn clean_cache(max_age: u128) -> Result<()> {
     info!("cleaning cache with max_age: {:?}", max_age);
 
     if max_age == 0 {
@@ -687,7 +575,7 @@ fn clean_cache(max_age: u64) -> Result<()> {
     for child in fs::read_dir(cache_dir)? {
         let child = child?;
         let path = child.path();
-        if path.is_file_polyfill() {
+        if path.is_file() {
             continue;
         }
 
@@ -814,30 +702,14 @@ fn gen_pkg_and_compile(input: &Input, action: &InputAction) -> Result<()> {
             &meta,
         )?;
 
-        #[cfg(feature = "suppress-cargo-output")]
         macro_rules! get_status {
             ($cmd:expr) => {
-                // `try!` doesn't work here on <=1.12.
-                (match util::suppress_child_output(
-                    &mut $cmd,
-                    ::std::time::Duration::from_millis(CARGO_OUTPUT_TIMEOUT),
-                ) {
-                    Ok(v) => v,
-                    Err(e) => return Err(e),
-                })
-                .status()
-            };
-        }
-
-        #[cfg(not(feature = "suppress-cargo-output"))]
-        macro_rules! get_status {
-            ($cmd:expr) => {
-                $cmd.status()
+                util::suppress_child_output(&mut $cmd)?.status()
             };
         }
 
         let compile_err = get_status!(cmd)
-            .map_err(|e| Into::<MainError>::into(e))
+            .map_err(Into::<MainError>::into)
             .and_then(|st| match st.code() {
                 Some(0) => Ok(()),
                 Some(st) => Err(format!("cargo failed with status {}", st).into()),
@@ -949,13 +821,13 @@ The metadata here serves two purposes:
 1. It records everything necessary for compilation and execution of a package.
 2. It records everything that must be exactly the same in order for a cached executable to still be valid, in addition to the content hash.
 */
-#[derive(Clone, Debug, Eq, PartialEq, RustcDecodable, RustcEncodable)]
+#[derive(Clone, Debug, Eq, PartialEq, Serialize, Deserialize)]
 struct PackageMetadata {
     /// Path to the script file.
     path: Option<String>,
 
     /// Last-modified timestamp for script file.
-    modified: Option<u64>,
+    modified: Option<u128>,
 
     /// Template used.
     template: Option<String>,
@@ -993,30 +865,27 @@ fn decide_action_for(
     input: &Input,
     deps: Vec<(String, String)>,
     prelude: Vec<String>,
-    debug: bool,
-    pkg_path: Option<String>,
-    gen_pkg_only: bool,
-    build_only: bool,
-    force: bool,
-    features: Option<String>,
-    use_bincache: Option<bool>,
-    build_kind: BuildKind,
+    args: &Args,
 ) -> Result<InputAction> {
-    let (pkg_path, using_cache) = pkg_path.map(|p| (p.into(), false)).unwrap_or_else(|| {
-        // This can't fail.  Seriously, we're *fucked* if we can't work this out.
-        let cache_path = get_script_cache_path().unwrap();
-        info!("cache_path: {:?}", cache_path);
+    let (pkg_path, using_cache) = args
+        .pkg_path
+        .as_ref()
+        .map(|p| (p.into(), false))
+        .unwrap_or_else(|| {
+            // This can't fail.  Seriously, we're *fucked* if we can't work this out.
+            let cache_path = get_script_cache_path().unwrap();
+            info!("cache_path: {:?}", cache_path);
 
-        let id = {
-            let deps_iter = deps.iter().map(|&(ref n, ref v)| (n as &str, v as &str));
+            let id = {
+                let deps_iter = deps.iter().map(|&(ref n, ref v)| (n as &str, v as &str));
 
-            // Again, also fucked if we can't work this out.
-            input.compute_id(deps_iter).unwrap()
-        };
-        info!("id: {:?}", id);
+                // Again, also fucked if we can't work this out.
+                input.compute_id(deps_iter).unwrap()
+            };
+            info!("id: {:?}", id);
 
-        (cache_path.join(&id), true)
-    });
+            (cache_path.join(&id), true)
+        });
     info!("pkg_path: {:?}", pkg_path);
     info!("using_cache: {:?}", using_cache);
 
@@ -1024,8 +893,8 @@ fn decide_action_for(
     let (mani_str, script_str) = manifest::split_input(input, &deps, &prelude)?;
 
     // Forcibly override some flags based on build kind.
-    let (debug, force, build_only) = match build_kind {
-        BuildKind::Normal => (debug, force, build_only),
+    let (debug, force, build_only) = match args.build_kind {
+        BuildKind::Normal => (args.debug, args.force, args.build_only),
         BuildKind::Test => (true, false, false),
         BuildKind::Bench => (false, false, false),
     };
@@ -1040,13 +909,13 @@ fn decide_action_for(
             Input::Loop(..) => (None, None, None),
         };
         PackageMetadata {
-            path: path,
+            path,
             modified: mtime,
             template: template.map(Into::into),
-            debug: debug,
-            deps: deps,
-            prelude: prelude,
-            features: features,
+            debug,
+            deps,
+            prelude,
+            features: args.features.clone(),
             manifest_hash: hash_str(&mani_str),
             script_hash: hash_str(&script_str),
         }
@@ -1059,14 +928,14 @@ fn decide_action_for(
         force_compile: force,
         emit_metadata: true,
         execute: !build_only,
-        pkg_path: pkg_path,
-        using_cache: using_cache,
-        use_bincache: use_bincache.unwrap_or(using_cache),
+        pkg_path,
+        using_cache,
+        use_bincache: args.use_bincache.unwrap_or(using_cache),
         metadata: input_meta,
         old_metadata: None,
         manifest: mani_str,
         script: script_str,
-        build_kind: build_kind,
+        build_kind: args.build_kind,
     };
 
     macro_rules! bail {
@@ -1079,7 +948,7 @@ fn decide_action_for(
     }
 
     // If we were told to only generate the package, we need to stop *now*
-    if gen_pkg_only {
+    if args.gen_pkg_only {
         bail!(compile: false, execute: false)
     }
 
@@ -1114,7 +983,7 @@ fn decide_action_for(
     Next test: does the executable exist at all?
     */
     let exe_exists = match get_exe_path(action.build_kind, &action.pkg_path) {
-        Ok(exe_path) => exe_path.is_file_polyfill(),
+        Ok(exe_path) => exe_path.is_file(),
         Err(_) => false,
     };
     if !exe_exists {
@@ -1129,7 +998,7 @@ fn decide_action_for(
     */
     if action.use_bincache {
         let exe_meta_hash_path = get_meta_hash_path(action.use_bincache, &action.pkg_path).unwrap();
-        if !exe_meta_hash_path.is_file_polyfill() {
+        if !exe_meta_hash_path.is_file() {
             info!("recompiling because: meta hash doesn't exist or isn't a file");
             bail!(compile: true, force_compile: true)
         }
@@ -1207,8 +1076,7 @@ where
         meta_file.read_to_string(&mut s).unwrap();
         s
     };
-    let meta: PackageMetadata =
-        rustc_serialize::json::decode(&meta_str).map_err(|err| err.to_string())?;
+    let meta: PackageMetadata = serde_json::from_str(&meta_str).map_err(|err| err.to_string())?;
 
     Ok(meta)
 }
@@ -1233,7 +1101,7 @@ where
     let meta_path = get_pkg_metadata_path(pkg_path);
     debug!("meta_path: {:?}", meta_path);
     let mut meta_file = fs::File::create(&meta_path)?;
-    let meta_str = rustc_serialize::json::encode(meta).map_err(|err| err.to_string())?;
+    let meta_str = serde_json::to_string(meta).map_err(|err| err.to_string())?;
     write!(&mut meta_file, "{}", meta_str)?;
     meta_file.flush()?;
     Ok(())
@@ -1296,7 +1164,7 @@ pub enum Input<'a> {
 
     The tuple members are: the name, absolute path, script contents, last modified time.
     */
-    File(&'a str, &'a Path, &'a str, u64),
+    File(&'a str, &'a Path, &'a str, u128),
 
     /**
     The input is an expression.
@@ -1317,8 +1185,8 @@ impl<'a> Input<'a> {
     /**
     Return the path to the script, if it has one.
     */
-    pub fn path(&self) -> Option<&Path> {
-        use Input::*;
+    pub const fn path(&self) -> Option<&Path> {
+        use crate::Input::*;
 
         match *self {
             File(_, path, _, _) => Some(path),
@@ -1332,8 +1200,8 @@ impl<'a> Input<'a> {
 
     Currently, nothing is done to ensure this, other than hoping *really hard* that we don't get fed some excessively bizzare input filename.
     */
-    pub fn safe_name(&self) -> &str {
-        use Input::*;
+    pub const fn safe_name(&self) -> &str {
+        use crate::Input::*;
 
         match *self {
             File(name, _, _, _) => name,
@@ -1389,9 +1257,9 @@ impl<'a> Input<'a> {
     where
         DepIt: IntoIterator<Item = (&'dep str, &'dep str)>,
     {
+        use crate::Input::*;
         use shaman::digest::Digest;
         use shaman::sha1::Sha1;
-        use Input::*;
 
         let hash_deps = || {
             let mut hasher = Sha1::new();
@@ -1428,7 +1296,7 @@ impl<'a> Input<'a> {
                 hasher.input_str(template.unwrap_or(""));
                 hasher.input_str(";");
 
-                hasher.input_str(&content);
+                hasher.input_str(content);
                 let mut digest = hasher.result_str();
                 digest.truncate(consts::ID_DIGEST_LEN_MAX);
 
@@ -1444,7 +1312,7 @@ impl<'a> Input<'a> {
                 hasher.input_str("count:");
                 hasher.input_str(if count { "true;" } else { "false;" });
 
-                hasher.input_str(&content);
+                hasher.input_str(content);
                 let mut digest = hasher.result_str();
                 digest.truncate(consts::ID_DIGEST_LEN_MAX);
 
@@ -1491,7 +1359,7 @@ where
     let mut file = fs::File::create(path)?;
     write!(&mut file, "{}", content)?;
     file.flush()?;
-    Ok(FileOverwrite::Changed { new_hash: new_hash })
+    Ok(FileOverwrite::Changed { new_hash })
 }
 
 /**
@@ -1511,7 +1379,12 @@ fn cargo(
     }
 
     if use_bincache {
-        cmd.env("CARGO_TARGET_DIR", get_binary_cache_path()?);
+        let script_specific_binary_cache = format!(
+            "{}/{}",
+            get_binary_cache_path()?.display(),
+            meta.sha1_hash()
+        );
+        cmd.env("CARGO_TARGET_DIR", script_specific_binary_cache);
     }
 
     // Block `--release` on `bench`.
@@ -1541,10 +1414,6 @@ fn cargo_target<P>(
 where
     P: AsRef<Path>,
 {
-    lazy_static! {
-        static ref VER_JSON_MSGS: Version = Version::parse("0.18.0").unwrap();
-    }
-
     trace!(
         "cargo_target(_, {:?}, {:?}, {:?}, _)",
         pkg_path.as_ref(),
@@ -1552,22 +1421,7 @@ where
         use_bincache
     );
 
-    let cargo_ver = cargo_version().err_tag("could not determine target filename")?;
-
-    let mut use_guess = false;
-    use_guess |= work_around_issue_50();
-    use_guess |= if cargo_ver < *VER_JSON_MSGS {
-        trace!(".. cargo {:?} is too old to support JSON output", cargo_ver);
-        true
-    } else {
-        false
-    };
-
-    let exe_path = if use_guess {
-        cargo_target_by_guess(input, use_bincache, pkg_path.as_ref(), meta)?
-    } else {
-        cargo_target_by_message(input, manifest, use_bincache, meta)?
-    };
+    let exe_path = cargo_target_by_message(input, manifest, use_bincache, meta)?;
 
     trace!(".. exe_path: {:?}", exe_path);
 
@@ -1586,52 +1440,13 @@ where
     Ok(exe_path)
 }
 
-/**
-Figures out where the output executable for the input should be by guessing.
-
-Depending on the configuration, this might not work.  On the other hand, this actually works (usually) prior to Cargo 0.18 (Rust 1.17).
-*/
-fn cargo_target_by_guess(
-    input: &Input,
-    use_bincache: bool,
-    pkg_path: &Path,
-    meta: &PackageMetadata,
-) -> Result<PathBuf> {
-    trace!(
-        "cargo_target_by_guess(_, {:?}, {:?}, _)",
-        use_bincache,
-        pkg_path
-    );
-
-    let profile = match meta.debug {
-        true => "debug",
-        false => "release",
-    };
-    let target_path = if use_bincache {
-        get_binary_cache_path()?
-    } else {
-        pkg_path.join("target")
-    };
-    let mut exe_path = target_path
-        .join(profile)
-        .join(&input.package_name())
-        .into_os_string();
-    exe_path.push(std::env::consts::EXE_SUFFIX);
-    Ok(exe_path.into())
-}
-
-/**
-Gets the path to the package's target file by parsing the output of `cargo build`.
-
-This only works on Cargo 0.18 (Rust 1.17) and higher.
-*/
+// Gets the path to the package's target file by parsing the output of `cargo build`.
 fn cargo_target_by_message(
     input: &Input,
     manifest: &str,
     use_bincache: bool,
     meta: &PackageMetadata,
 ) -> Result<PathBuf> {
-    use rustc_serialize::json;
     use std::io::{BufRead, BufReader};
 
     trace!(
@@ -1648,134 +1463,43 @@ fn cargo_target_by_message(
     trace!(".. cmd: {:?}", cmd);
 
     let mut child = cmd.spawn()?;
-    match child.wait()?.code() {
-        Some(0) => (),
-        Some(st) => {
-            return Err(format!(
-                "could not determine target filename: cargo exited with status {}",
-                st
-            )
-            .into())
-        }
-        None => {
-            return Err(
-                format!("could not determine target filename: cargo exited abnormally").into(),
-            )
-        }
-    }
 
-    let mut line = String::with_capacity(1024);
-    let mut stdout = BufReader::new(child.stdout.take().unwrap());
-    let null = json::Json::Null;
     let package_name = input.package_name();
 
-    let mut line_num = 0;
-    loop {
-        line_num += 1;
-        line.clear();
-        let bytes = stdout.read_line(&mut line)?;
-        trace!(".. line {}, {}b: {:?}", line_num, bytes, line);
-        if bytes == 0 {
-            return Err(
-                "could not determine target filename: did not find appropriate cargo message"
-                    .into(),
-            );
+    let stdout = BufReader::new(child.stdout.take().unwrap());
+    let mut lines = stdout.lines();
+
+    #[derive(Deserialize)]
+    struct Target {
+        name: String,
+    }
+
+    #[derive(Deserialize)]
+    struct Line {
+        reason: String,
+        target: Target,
+        filenames: Vec<PathBuf>,
+    }
+
+    while let Some(Ok(line)) = lines.next() {
+        if let Ok(mut l) = serde_json::from_str::<Line>(&line).map_err(Box::new) {
+            if l.reason == "compiler-artifact" && l.target.name == package_name {
+                let _ = child.kill();
+                return Ok(l.filenames.swap_remove(0));
+            }
         }
+    }
 
-        let msg = json::Json::from_str(line.trim()).map_err(Box::new)?;
-
-        // Is this the message we're looking for?
-        if msg.find("reason").unwrap_or(&null).as_string() != Some("compiler-artifact") {
-            trace!("   couldn't find `compiler-artifact`");
-            continue;
-        }
-        if msg
-            .find_path(&["target", "name"])
-            .unwrap_or(&null)
-            .as_string()
-            != Some(&package_name)
-        {
-            trace!(
-                "   couldn't find `target.name`, or it wasn't {:?}",
-                package_name
-            );
-            continue;
-        }
-
-        // Looks like it; grab the path.
-        let exe_path = msg
-            .find_path(&["filenames"])
-            .expect("could not find `filenames` in json message")
-            .as_array()
-            .expect("`filenames` in json message was not an array")[0]
-            .as_string()
-            .expect("`filenames[0]` in json message was not a string");
-
-        return Ok(exe_path.into());
-    }
-}
-
-/**
-Get the version of the currently active cargo.
-*/
-fn cargo_version() -> Result<Version> {
-    use regex::Regex;
-
-    lazy_static! {
-        static ref RE_VERSION: Regex = Regex::new(r#"^cargo[ -](\S+)"#).unwrap();
-    }
-
-    let mut cmd = Command::new("cargo");
-    cmd.arg("-V");
-
-    let child = cmd.output()?;
-    match child.status.code() {
-        Some(0) => (),
-        Some(st) => {
-            return Err(format!(
-                "could not determine cargo version: cargo exited with status {}",
-                st
-            )
-            .into())
-        }
-        None => {
-            return Err(
-                format!("could not determine cargo version: cargo exited abnormally").into(),
-            )
-        }
-    }
-
-    let stdout = String::from_utf8_lossy(&child.stdout);
-    let m = match RE_VERSION.captures(&stdout) {
-        Some(m) => m,
-        None => {
-            return Err(
-                format!("could not determine cargo version: output did not match expected").into(),
-            )
-        }
-    };
-
-    let ver = m.get(1).unwrap();
-    Ok(Version::parse(ver.as_str()).map_err(Box::new)?)
-}
-
-/**
-Do we need to work around [issue #50](https://github.com/DanielKeep/cargo-script/issues/50)?
-
-Sometimes, `cargo-script` will hang when trying to read the JSON output of `cargo build`.
-*/
-fn work_around_issue_50() -> bool {
-    let suffers = cfg!(issue_50);
-    let ignored = std::env::var_os("CARGO_SCRIPT_IGNORE_ISSUE_50").is_some();
-    match (suffers, ignored) {
-        (true, true) => {
-            trace!(".. issue 50 relevant, but ignored");
-            false
-        }
-        (true, false) => {
-            trace!(".. working around issue 50");
-            true
-        }
-        (false, _) => false,
+    match child.wait()?.code() {
+        Some(st) => Err(format!(
+            "could not determine target filename: cargo exited with status {}",
+            st
+        )
+        .into()),
+        None => Err(
+            "could not determine target filename: cargo exited abnormally"
+                .to_string()
+                .into(),
+        ),
     }
 }
