@@ -10,22 +10,21 @@ or distributed except according to those terms.
 /*!
 This module is concerned with how `cargo-script` extracts the manfiest from a script file.
 */
-extern crate hoedown;
+extern crate pulldown_cmark;
 extern crate regex;
 
+use self::regex::Regex;
 use std::collections::HashMap;
 use std::path::Path;
-use self::regex::Regex;
-use toml;
 
-use consts;
-use error::{Blame, Result};
-use templates;
-use Input;
+use crate::consts;
+use crate::error::{Blame, MainError, Result};
+use crate::templates;
+use crate::Input;
 
 lazy_static! {
-    static ref RE_SHORT_MANIFEST: Regex = Regex::new(
-        r"^(?i)\s*//\s*cargo-deps\s*:(.*?)(\r\n|\n)").unwrap();
+    static ref RE_SHORT_MANIFEST: Regex =
+        Regex::new(r"^(?i)\s*//\s*cargo-deps\s*:(.*?)(\r\n|\n)").unwrap();
     static ref RE_MARGIN: Regex = Regex::new(r"^\s*\*( |$)").unwrap();
     static ref RE_SPACE: Regex = Regex::new(r"^(\s+)").unwrap();
     static ref RE_NESTING: Regex = Regex::new(r"/\*|\*/").unwrap();
@@ -47,33 +46,36 @@ Splits input into a complete Cargo manifest and unadultered Rust source.
 
 Unless we have prelude items to inject, in which case it will be *slightly* adulterated.
 */
-pub fn split_input(input: &Input, deps: &[(String, String)], prelude_items: &[String]) -> Result<(String, String)> {
+pub fn split_input(
+    input: &Input,
+    deps: &[(String, String)],
+    prelude_items: &[String],
+) -> Result<(String, String)> {
     let template_buf;
     let (part_mani, source, template, sub_prelude) = match *input {
         Input::File(_, _, content, _) => {
             assert_eq!(prelude_items.len(), 0);
             let content = strip_hashbang(content);
-            let (manifest, source) = find_embedded_manifest(content)
-                .unwrap_or((Manifest::Toml(""), content));
+            let (manifest, source) =
+                find_embedded_manifest(content).unwrap_or((Manifest::Toml(""), content));
 
             (manifest, source, templates::get_template("file")?, false)
-        },
-        Input::Expr("meaning-of-life", None) | Input::Expr("meaning_of_life", None) => {
-            (Manifest::Toml(""), r#"
-                println!("42");
-                std::process::exit(42);
-            "#, templates::get_template("expr")?, true)
-        },
+        }
         Input::Expr(content, template) => {
             template_buf = templates::get_template(template.unwrap_or("expr"))?;
             let (manifest, template_src) = find_embedded_manifest(&template_buf)
                 .unwrap_or((Manifest::Toml(""), &template_buf));
             (manifest, content, template_src.into(), true)
-        },
+        }
         Input::Loop(content, count) => {
             let templ = if count { "loop-count" } else { "loop" };
-            (Manifest::Toml(""), content, templates::get_template(templ)?, true)
-        },
+            (
+                Manifest::Toml(""),
+                content,
+                templates::get_template(templ)?,
+                true,
+            )
+        }
     };
 
     let mut prelude_str;
@@ -81,13 +83,11 @@ pub fn split_input(input: &Input, deps: &[(String, String)], prelude_items: &[St
     subs.insert(consts::SCRIPT_BODY_SUB, &source[..]);
 
     if sub_prelude {
-        prelude_str = String::with_capacity(prelude_items
-            .iter()
-            .map(|i| i.len() + 1)
-            .sum::<usize>());
+        prelude_str =
+            String::with_capacity(prelude_items.iter().map(|i| i.len() + 1).sum::<usize>());
         for i in prelude_items {
             prelude_str.push_str(i);
-            prelude_str.push_str("\n");
+            prelude_str.push('\n');
         }
         subs.insert(consts::SCRIPT_PRELUDE_SUB, &prelude_str[..]);
     }
@@ -120,7 +120,9 @@ pub fn split_input(input: &Input, deps: &[(String, String)], prelude_items: &[St
 #[test]
 fn test_split_input() {
     macro_rules! si {
-        ($i:expr) => (split_input(&$i, &[], &[]).ok())
+        ($i:expr) => {
+            split_input(&$i, &[], &[]).ok()
+        };
     }
 
     let dummy_path: ::std::path::PathBuf = "p".into();
@@ -128,14 +130,15 @@ fn test_split_input() {
     let f = |c| Input::File("n", &dummy_path, c, 0);
 
     macro_rules! r {
-        ($m:expr, $r:expr) => (Some(($m.into(), $r.into())));
+        ($m:expr, $r:expr) => {
+            Some(($m.into(), $r.into()))
+        };
     }
 
-    assert_eq!(si!(f(
-r#"fn main() {}"#
-        )),
+    assert_eq!(
+        si!(f(r#"fn main() {}"#)),
         r!(
-r#"[[bin]]
+            r#"[[bin]]
 name = "n"
 path = "n.rs"
 
@@ -143,22 +146,22 @@ path = "n.rs"
 
 [package]
 authors = ["Anonymous"]
+edition = "2018"
 name = "n"
 version = "0.1.0"
 "#,
-r#"fn main() {}"#
+            r#"fn main() {}"#
         )
     );
 
     // Ensure removed prefix manifests don't work.
-    assert_eq!(si!(f(
-r#"
+    assert_eq!(
+        si!(f(r#"
 ---
 fn main() {}
-"#
-        )),
+"#)),
         r!(
-r#"[[bin]]
+            r#"[[bin]]
 name = "n"
 path = "n.rs"
 
@@ -166,25 +169,25 @@ path = "n.rs"
 
 [package]
 authors = ["Anonymous"]
+edition = "2018"
 name = "n"
 version = "0.1.0"
 "#,
-r#"
+            r#"
 ---
 fn main() {}
 "#
         )
     );
 
-    assert_eq!(si!(f(
-r#"[dependencies]
+    assert_eq!(
+        si!(f(r#"[dependencies]
 time="0.1.25"
 ---
 fn main() {}
-"#
-        )),
+"#)),
         r!(
-r#"[[bin]]
+            r#"[[bin]]
 name = "n"
 path = "n.rs"
 
@@ -192,10 +195,11 @@ path = "n.rs"
 
 [package]
 authors = ["Anonymous"]
+edition = "2018"
 name = "n"
 version = "0.1.0"
 "#,
-r#"[dependencies]
+            r#"[dependencies]
 time="0.1.25"
 ---
 fn main() {}
@@ -203,14 +207,13 @@ fn main() {}
         )
     );
 
-    assert_eq!(si!(f(
-r#"
+    assert_eq!(
+        si!(f(r#"
 // Cargo-Deps: time="0.1.25"
 fn main() {}
-"#
-        )),
+"#)),
         r!(
-r#"[[bin]]
+            r#"[[bin]]
 name = "n"
 path = "n.rs"
 
@@ -219,24 +222,24 @@ time = "0.1.25"
 
 [package]
 authors = ["Anonymous"]
+edition = "2018"
 name = "n"
 version = "0.1.0"
 "#,
-r#"
+            r#"
 // Cargo-Deps: time="0.1.25"
 fn main() {}
 "#
         )
     );
 
-    assert_eq!(si!(f(
-r#"
+    assert_eq!(
+        si!(f(r#"
 // Cargo-Deps: time="0.1.25", libc="0.2.5"
 fn main() {}
-"#
-        )),
+"#)),
         r!(
-r#"[[bin]]
+            r#"[[bin]]
 name = "n"
 path = "n.rs"
 
@@ -246,18 +249,19 @@ time = "0.1.25"
 
 [package]
 authors = ["Anonymous"]
+edition = "2018"
 name = "n"
 version = "0.1.0"
 "#,
-r#"
+            r#"
 // Cargo-Deps: time="0.1.25", libc="0.2.5"
 fn main() {}
 "#
         )
     );
 
-    assert_eq!(si!(f(
-r#"
+    assert_eq!(
+        si!(f(r#"
 /*!
 Here is a manifest:
 
@@ -267,10 +271,9 @@ time = "0.1.25"
 ```
 */
 fn main() {}
-"#
-        )),
+"#)),
         r!(
-r#"[[bin]]
+            r#"[[bin]]
 name = "n"
 path = "n.rs"
 
@@ -279,10 +282,11 @@ time = "0.1.25"
 
 [package]
 authors = ["Anonymous"]
+edition = "2018"
 name = "n"
 version = "0.1.0"
 "#,
-r#"
+            r#"
 /*!
 Here is a manifest:
 
@@ -303,29 +307,39 @@ Returns a slice of the input string with the leading hashbang, if there is one,
 fn strip_hashbang(s: &str) -> &str {
     match RE_HASHBANG.find(s) {
         Some(m) => &s[m.end()..],
-        None => s
+        None => s,
     }
 }
 
 #[test]
 fn test_strip_hashbang() {
-    assert_eq!(strip_hashbang("\
+    assert_eq!(
+        strip_hashbang(
+            "\
 #!/usr/bin/env run-cargo-script
 and the rest
 \
-        "), "\
+        "
+        ),
+        "\
 and the rest
 \
-        ");
-    assert_eq!(strip_hashbang("\
+        "
+    );
+    assert_eq!(
+        strip_hashbang(
+            "\
 #![thingy]
 and the rest
 \
-        "), "\
+        "
+        ),
+        "\
 #![thingy]
 and the rest
 \
-        ");
+        "
+    );
 }
 
 /**
@@ -343,18 +357,23 @@ enum Manifest<'s> {
 }
 
 impl<'s> Manifest<'s> {
-    pub fn into_toml(self) -> Result<toml::Table> {
+    pub fn into_toml(self) -> Result<toml::value::Table> {
         use self::Manifest::*;
         match self {
-            Toml(s) => Ok(toml::Parser::new(s).parse()
-                .ok_or("could not parse embedded manifest")?),
-            TomlOwned(ref s) => Ok(toml::Parser::new(s).parse()
-                .ok_or("could not parse embedded manifest")?),
+            Toml(s) => toml::from_str(s),
+            TomlOwned(ref s) => toml::from_str(s),
             DepList(s) => Manifest::dep_list_to_toml(s),
         }
+        .map_err(|e| {
+            MainError::Tag(
+                Blame::Human,
+                "could not parse embedded manifest".into(),
+                Box::new(MainError::Other(Blame::Internal, Box::new(e))),
+            )
+        })
     }
 
-    fn dep_list_to_toml(s: &str) -> Result<toml::Table> {
+    fn dep_list_to_toml(s: &str) -> ::std::result::Result<toml::value::Table, toml::de::Error> {
         let mut r = String::new();
         r.push_str("[dependencies]\n");
         for dep in s.trim().split(',') {
@@ -362,8 +381,8 @@ impl<'s> Manifest<'s> {
             match dep.contains('=') {
                 true => {
                     r.push_str(dep);
-                    r.push_str("\n");
-                },
+                    r.push('\n');
+                }
                 false => {
                     r.push_str(dep);
                     r.push_str("=\"*\"\n");
@@ -371,8 +390,7 @@ impl<'s> Manifest<'s> {
             }
         }
 
-        Ok(toml::Parser::new(&r).parse()
-            .ok_or("could not parse embedded manifest")?)
+        toml::from_str(&r)
     }
 }
 
@@ -382,8 +400,7 @@ Locates a manifest embedded in Rust source.
 Returns `Some((manifest, source))` if it finds a manifest, `None` otherwise.
 */
 fn find_embedded_manifest(s: &str) -> Option<(Manifest, &str)> {
-    find_short_comment_manifest(s)
-        .or_else(|| find_code_block_manifest(s))
+    find_short_comment_manifest(s).or_else(|| find_code_block_manifest(s))
 }
 
 #[test]
@@ -394,125 +411,140 @@ fn test_find_embedded_manifest() {
 
     assert_eq!(fem("fn main() {}"), None);
 
-    assert_eq!(fem(
-"
+    assert_eq!(
+        fem("
 fn main() {}
 "),
-    None);
+        None
+    );
 
     // Ensure removed prefix manifests don't work.
-    assert_eq!(fem(
-r#"
+    assert_eq!(
+        fem(r#"
 ---
 fn main() {}
 "#),
-    None);
+        None
+    );
 
-    assert_eq!(fem(
-"[dependencies]
+    assert_eq!(
+        fem("[dependencies]
 time = \"0.1.25\"
 ---
 fn main() {}
 "),
-    None);
+        None
+    );
 
-    assert_eq!(fem(
-"[dependencies]
+    assert_eq!(
+        fem("[dependencies]
 time = \"0.1.25\"
 
 fn main() {}
 "),
-    None);
+        None
+    );
 
     // Make sure we aren't just grabbing the *last* line.
-    assert_eq!(fem(
-"[dependencies]
+    assert_eq!(
+        fem("[dependencies]
 time = \"0.1.25\"
 
 fn main() {
     println!(\"Hi!\");
 }
 "),
-    None);
+        None
+    );
 
-    assert_eq!(fem(
-"// cargo-deps: time=\"0.1.25\"
+    assert_eq!(
+        fem("// cargo-deps: time=\"0.1.25\"
 fn main() {}
 "),
-    Some((
-DepList(" time=\"0.1.25\""),
-"// cargo-deps: time=\"0.1.25\"
+        Some((
+            DepList(" time=\"0.1.25\""),
+            "// cargo-deps: time=\"0.1.25\"
 fn main() {}
 "
-    )));
+        ))
+    );
 
-    assert_eq!(fem(
-"// cargo-deps: time=\"0.1.25\", libc=\"0.2.5\"
+    assert_eq!(
+        fem("// cargo-deps: time=\"0.1.25\", libc=\"0.2.5\"
 fn main() {}
 "),
-    Some((
-DepList(" time=\"0.1.25\", libc=\"0.2.5\""),
-"// cargo-deps: time=\"0.1.25\", libc=\"0.2.5\"
+        Some((
+            DepList(" time=\"0.1.25\", libc=\"0.2.5\""),
+            "// cargo-deps: time=\"0.1.25\", libc=\"0.2.5\"
 fn main() {}
 "
-    )));
+        ))
+    );
 
-    assert_eq!(fem(
-"
+    assert_eq!(
+        fem("
   // cargo-deps: time=\"0.1.25\"  \n\
 fn main() {}
 "),
-    Some((
-DepList(" time=\"0.1.25\"  "),
-"
+        Some((
+            DepList(" time=\"0.1.25\"  "),
+            "
   // cargo-deps: time=\"0.1.25\"  \n\
 fn main() {}
 "
-    )));
+        ))
+    );
 
-    assert_eq!(fem(
-"/* cargo-deps: time=\"0.1.25\" */
+    assert_eq!(
+        fem("/* cargo-deps: time=\"0.1.25\" */
 fn main() {}
 "),
-    None);
+        None
+    );
 
-    assert_eq!(fem(
-r#"//! [dependencies]
+    assert_eq!(
+        fem(r#"//! [dependencies]
 //! time = "0.1.25"
 fn main() {}
 "#),
-    None);
+        None
+    );
 
-    assert_eq!(fem(
-r#"//! ```Cargo
+    assert_eq!(
+        fem(r#"//! ```Cargo
 //! [dependencies]
 //! time = "0.1.25"
 //! ```
 fn main() {}
 "#),
-    Some((
-TomlOwned(r#"[dependencies]
+        Some((
+            TomlOwned(
+                r#"[dependencies]
 time = "0.1.25"
-"#.into()),
-r#"//! ```Cargo
+"#
+                .into()
+            ),
+            r#"//! ```Cargo
 //! [dependencies]
 //! time = "0.1.25"
 //! ```
 fn main() {}
 "#
-    )));
+        ))
+    );
 
-    assert_eq!(fem(
-r#"/*!
+    assert_eq!(
+        fem(r#"/*!
 [dependencies]
 time = "0.1.25"
 */
 fn main() {}
 "#),
-    None);
+        None
+    );
 
-    assert_eq!(fem(
-r#"/*!
+    assert_eq!(
+        fem(r#"/*!
 ```Cargo
 [dependencies]
 time = "0.1.25"
@@ -520,11 +552,14 @@ time = "0.1.25"
 */
 fn main() {}
 "#),
-    Some((
-TomlOwned(r#"[dependencies]
+        Some((
+            TomlOwned(
+                r#"[dependencies]
 time = "0.1.25"
-"#.into()),
-r#"/*!
+"#
+                .into()
+            ),
+            r#"/*!
 ```Cargo
 [dependencies]
 time = "0.1.25"
@@ -532,19 +567,21 @@ time = "0.1.25"
 */
 fn main() {}
 "#
-    )));
+        ))
+    );
 
-    assert_eq!(fem(
-r#"/*!
+    assert_eq!(
+        fem(r#"/*!
  * [dependencies]
  * time = "0.1.25"
  */
 fn main() {}
 "#),
-    None);
+        None
+    );
 
-    assert_eq!(fem(
-r#"/*!
+    assert_eq!(
+        fem(r#"/*!
  * ```Cargo
  * [dependencies]
  * time = "0.1.25"
@@ -552,11 +589,14 @@ r#"/*!
  */
 fn main() {}
 "#),
-    Some((
-TomlOwned(r#"[dependencies]
+        Some((
+            TomlOwned(
+                r#"[dependencies]
 time = "0.1.25"
-"#.into()),
-r#"/*!
+"#
+                .into()
+            ),
+            r#"/*!
  * ```Cargo
  * [dependencies]
  * time = "0.1.25"
@@ -564,7 +604,8 @@ r#"/*!
  */
 fn main() {}
 "#
-    )));
+        ))
+    );
 }
 
 /**
@@ -577,7 +618,7 @@ fn find_short_comment_manifest(s: &str) -> Option<(Manifest, &str)> {
     let re = &*RE_SHORT_MANIFEST;
     if let Some(cap) = re.captures(s) {
         if let Some(m) = cap.get(1) {
-            return Some((Manifest::DepList(m.as_str()), &s[..]))
+            return Some((Manifest::DepList(m.as_str()), &s[..]));
         }
     }
     None
@@ -599,16 +640,16 @@ fn find_code_block_manifest(s: &str) -> Option<(Manifest, &str)> {
     let start = match RE_CRATE_COMMENT.captures(s) {
         Some(cap) => match cap.get(1) {
             Some(m) => m.start(),
-            None => return None
+            None => return None,
         },
-        None => return None
+        None => return None,
     };
 
     let comment = match extract_comment(&s[start..]) {
         Ok(s) => s,
         Err(err) => {
             error!("error slicing comment: {}", err);
-            return None
+            return None;
         }
     };
 
@@ -621,62 +662,56 @@ fn find_code_block_manifest(s: &str) -> Option<(Manifest, &str)> {
 Extracts the first `Cargo` fenced code block from a chunk of Markdown.
 */
 fn scrape_markdown_manifest(content: &str) -> Result<Option<String>> {
-    use self::hoedown::{Buffer, Markdown, Render};
+    use self::pulldown_cmark::{CodeBlockKind, Event, Options, Parser, Tag};
 
-    // To match librustdoc/html/markdown.rs, HOEDOWN_EXTENSIONS.
-    let exts
-        = hoedown::NO_INTRA_EMPHASIS
-        | hoedown::TABLES
-        | hoedown::FENCED_CODE
-        | hoedown::AUTOLINK
-        | hoedown::STRIKETHROUGH
-        | hoedown::SUPERSCRIPT
-        | hoedown::FOOTNOTES;
+    // To match librustdoc/html/markdown.rs, opts.
+    let exts = Options::ENABLE_TABLES | Options::ENABLE_FOOTNOTES;
 
-    let md = Markdown::new(&content).extensions(exts);
+    let md = Parser::new_ext(content, exts);
 
-    struct ManifestScraper {
-        seen_manifest: bool,
-    }
+    let mut found = false;
+    let mut output = None;
 
-    impl Render for ManifestScraper {
-        fn code_block(&mut self, output: &mut Buffer, text: Option<&Buffer>, lang: Option<&Buffer>) {
-            let lang = lang.map(|b| b.to_str().unwrap()).unwrap_or("");
-
-            if !self.seen_manifest && lang.eq_ignore_ascii_case("cargo") {
-                // Pass it through.
-                info!("found code block manifest");
-                if let Some(text) = text {
-                    output.pipe(text);
-                }
-                self.seen_manifest = true;
+    for item in md {
+        match item {
+            Event::Start(Tag::CodeBlock(CodeBlockKind::Fenced(ref info)))
+                if info.to_lowercase() == "cargo" && output.is_none() =>
+            {
+                found = true;
             }
+            Event::Text(ref text) if found => {
+                let s = output.get_or_insert(String::new());
+                s.push_str(&*text);
+            }
+            Event::End(Tag::CodeBlock(_)) if found => {
+                found = false;
+            }
+            _ => (),
         }
     }
 
-    let mut ms = ManifestScraper { seen_manifest: false };
-    let mani_buf = ms.render(&md);
-
-    if !ms.seen_manifest { return Ok(None) }
-    mani_buf.to_str().map(|s| Some(s.into()))
-        .map_err(|_| "error decoding manifest as UTF-8".into())
+    Ok(output)
 }
 
 #[test]
 fn test_scrape_markdown_manifest() {
     macro_rules! smm {
-        ($c:expr) => (scrape_markdown_manifest($c).map_err(|e| e.to_string()));
+        ($c:expr) => {
+            scrape_markdown_manifest($c).map_err(|e| e.to_string())
+        };
     }
 
-    assert_eq!(smm!(
-r#"There is no manifest in this comment.
+    assert_eq!(
+        smm!(
+            r#"There is no manifest in this comment.
 "#
         ),
-Ok(None)
+        Ok(None)
     );
 
-    assert_eq!(smm!(
-r#"There is no manifest in this comment.
+    assert_eq!(
+        smm!(
+            r#"There is no manifest in this comment.
 
 ```
 This is not a manifest.
@@ -689,23 +724,28 @@ println!("Nor is this.");
     Or this.
 "#
         ),
-Ok(None)
+        Ok(None)
     );
 
-    assert_eq!(smm!(
-r#"This is a manifest:
+    assert_eq!(
+        smm!(
+            r#"This is a manifest:
 
 ```cargo
 dependencies = { time = "*" }
 ```
 "#
         ),
-Ok(Some(r#"dependencies = { time = "*" }
-"#.into()))
+        Ok(Some(
+            r#"dependencies = { time = "*" }
+"#
+            .into()
+        ))
     );
 
-    assert_eq!(smm!(
-r#"This is *not* a manifest:
+    assert_eq!(
+        smm!(
+            r#"This is *not* a manifest:
 
 ```
 He's lying, I'm *totally* a manifest!
@@ -718,12 +758,16 @@ dependencies = { time = "*" }
 ```
 "#
         ),
-Ok(Some(r#"dependencies = { time = "*" }
-"#.into()))
+        Ok(Some(
+            r#"dependencies = { time = "*" }
+"#
+            .into()
+        ))
     );
 
-    assert_eq!(smm!(
-r#"This is a manifest:
+    assert_eq!(
+        smm!(
+            r#"This is a manifest:
 
 ```cargo
 dependencies = { time = "*" }
@@ -736,8 +780,11 @@ dependencies = { explode = true }
 ```
 "#
         ),
-Ok(Some(r#"dependencies = { time = "*" }
-"#.into()))
+        Ok(Some(
+            r#"dependencies = { time = "*" }
+"#
+            .into()
+        ))
     );
 }
 
@@ -749,7 +796,7 @@ fn extract_comment(s: &str) -> Result<String> {
 
     fn n_leading_spaces(s: &str, n: usize) -> Result<()> {
         if !s.chars().take(n).all(|c| c == ' ') {
-            return Err(format!("leading {:?} chars aren't all spaces: {:?}", n, s).into())
+            return Err(format!("leading {:?} chars aren't all spaces: {:?}", n, s).into());
         }
         Ok(())
     }
@@ -777,46 +824,45 @@ fn extract_comment(s: &str) -> Result<String> {
         let mut depth: u32 = 1;
 
         for line in s.lines() {
-            if depth == 0 { break }
+            if depth == 0 {
+                break;
+            }
 
             // Update nesting and look for end-of-comment.
             let mut end_of_comment = None;
 
-            for (end, marker) in {
-                nesting_re.find_iter(line)
-                    .map(|m| (m.start(), m.as_str()))
-            } {
+            for (end, marker) in nesting_re.find_iter(line).map(|m| (m.start(), m.as_str())) {
                 match (marker, depth) {
                     ("/*", _) => depth += 1,
                     ("*/", 1) => {
                         end_of_comment = Some(end);
                         depth = 0;
                         break;
-                    },
+                    }
                     ("*/", _) => depth -= 1,
-                    _ => panic!("got a comment marker other than /* or */")
+                    _ => panic!("got a comment marker other than /* or */"),
                 }
             }
 
             let line = end_of_comment.map(|end| &line[..end]).unwrap_or(line);
 
             // Detect and strip margin.
-            margin = margin
-                .or_else(|| margin_re.find(line)
-                    .and_then(|m| Some(m.as_str())));
+            margin = margin.or_else(|| margin_re.find(line).map(|m| m.as_str()));
 
             let line = if let Some(margin) = margin {
-                let end = line.char_indices().take(margin.len())
-                    .map(|(i,c)| i + c.len_utf8()).last().unwrap_or(0);
+                let end = line
+                    .char_indices()
+                    .take(margin.len())
+                    .map(|(i, c)| i + c.len_utf8())
+                    .last()
+                    .unwrap_or(0);
                 &line[end..]
             } else {
                 line
             };
 
             // Detect and strip leading indentation.
-            leading_space = leading_space
-                .or_else(|| space_re.find(line)
-                    .map(|m| m.end()));
+            leading_space = leading_space.or_else(|| space_re.find(line).map(|m| m.end()));
 
             /*
             Make sure we have only leading spaces.
@@ -834,7 +880,7 @@ fn extract_comment(s: &str) -> Result<String> {
             r.push_str(line);
 
             // `lines` removes newlines.  Ideally, it wouldn't do that, but hopefully this shouldn't cause any *real* problems.
-            r.push_str("\n");
+            r.push('\n');
         }
 
         Ok(r)
@@ -852,14 +898,16 @@ fn extract_comment(s: &str) -> Result<String> {
             // Strip leading comment marker.
             let content = match comment_re.find(line) {
                 Some(m) => &line[m.end()..],
-                None => break
+                None => break,
             };
 
             // Detect and strip leading indentation.
-            leading_space = leading_space
-                .or_else(|| space_re.captures(content)
+            leading_space = leading_space.or_else(|| {
+                space_re
+                    .captures(content)
                     .and_then(|c| c.get(1))
-                    .map(|m| m.end()));
+                    .map(|m| m.end())
+            });
 
             /*
             Make sure we have only leading spaces.
@@ -877,7 +925,7 @@ fn extract_comment(s: &str) -> Result<String> {
             r.push_str(content);
 
             // `lines` removes newlines.  Ideally, it wouldn't do that, but hopefully this shouldn't cause any *real* problems.
-            r.push_str("\n");
+            r.push('\n');
         }
 
         Ok(r)
@@ -895,17 +943,15 @@ fn extract_comment(s: &str) -> Result<String> {
 #[test]
 fn test_extract_comment() {
     macro_rules! ec {
-        ($s:expr) => (extract_comment($s).map_err(|e| e.to_string()))
+        ($s:expr) => {
+            extract_comment($s).map_err(|e| e.to_string())
+        };
     }
 
-    assert_eq!(ec!(
-r#"fn main () {}"#
-        ),
-Err("no doc comment found".into())
-    );
+    assert_eq!(ec!(r#"fn main () {}"#), Err("no doc comment found".into()));
 
-    assert_eq!(ec!(
-r#"/*!
+    assert_eq!(
+        ec!(r#"/*!
 Here is a manifest:
 
 ```cargo
@@ -914,9 +960,8 @@ time = "*"
 ```
 */
 fn main() {}
-"#
-        ),
-Ok(r#"
+"#),
+        Ok(r#"
 Here is a manifest:
 
 ```cargo
@@ -924,11 +969,12 @@ Here is a manifest:
 time = "*"
 ```
 
-"#.into())
+"#
+        .into())
     );
 
-    assert_eq!(ec!(
-r#"/*!
+    assert_eq!(
+        ec!(r#"/*!
  * Here is a manifest:
  *
  * ```cargo
@@ -937,9 +983,8 @@ r#"/*!
  * ```
  */
 fn main() {}
-"#
-        ),
-Ok(r#"
+"#),
+        Ok(r#"
 Here is a manifest:
 
 ```cargo
@@ -947,33 +992,34 @@ Here is a manifest:
 time = "*"
 ```
 
-"#.into())
+"#
+        .into())
     );
 
-    assert_eq!(ec!(
-r#"//! Here is a manifest:
+    assert_eq!(
+        ec!(r#"//! Here is a manifest:
 //!
 //! ```cargo
 //! [dependencies]
 //! time = "*"
 //! ```
 fn main() {}
-"#
-        ),
-Ok(r#"Here is a manifest:
+"#),
+        Ok(r#"Here is a manifest:
 
 ```cargo
 [dependencies]
 time = "*"
 ```
-"#.into())
+"#
+        .into())
     );
 }
 
 /**
 Generates a default Cargo manifest for the given input.
 */
-fn default_manifest(input: &Input) -> Result<toml::Table> {
+fn default_manifest(input: &Input) -> Result<toml::value::Table> {
     let mani_str = {
         let pkg_name = input.package_name();
         let mut subs = HashMap::with_capacity(2);
@@ -981,31 +1027,44 @@ fn default_manifest(input: &Input) -> Result<toml::Table> {
         subs.insert(consts::MANI_FILE_SUB, &input.safe_name()[..]);
         templates::expand(consts::DEFAULT_MANIFEST, &subs)?
     };
-    toml::Parser::new(&mani_str).parse()
-        .ok_or("could not parse default manifest, somehow".into())
+    toml::from_str(&mani_str).map_err(|e| {
+        MainError::Tag(
+            Blame::Internal,
+            "could not parse default manifest".into(),
+            Box::new(MainError::Other(Blame::Internal, Box::new(e))),
+        )
+    })
 }
 
 /**
 Generates a partial Cargo manifest containing the specified dependencies.
 */
-fn deps_manifest(deps: &[(String, String)]) -> Result<toml::Table> {
+fn deps_manifest(deps: &[(String, String)]) -> Result<toml::value::Table> {
     let mut mani_str = String::new();
     mani_str.push_str("[dependencies]\n");
 
     for &(ref name, ref ver) in deps {
         mani_str.push_str(name);
-        mani_str.push_str("=");
+        mani_str.push('=');
 
         // We only want to quote the version if it *isn't* a table.
-        let quotes = match ver.starts_with("{") { true => "", false => "\"" };
+        let quotes = match ver.starts_with('{') {
+            true => "",
+            false => "\"",
+        };
         mani_str.push_str(quotes);
         mani_str.push_str(ver);
         mani_str.push_str(quotes);
-        mani_str.push_str("\n");
+        mani_str.push('\n');
     }
 
-    toml::Parser::new(&mani_str).parse()
-        .ok_or("could not parse dependency manifest".into())
+    toml::from_str(&mani_str).map_err(|e| {
+        MainError::Tag(
+            Blame::Human,
+            "could not parse dependency manifest".into(),
+            Box::new(MainError::Other(Blame::Internal, Box::new(e))),
+        )
+    })
 }
 
 /**
@@ -1013,38 +1072,41 @@ Given two Cargo manifests, merges the second *into* the first.
 
 Note that the "merge" in this case is relatively simple: only *top-level* tables are actually merged; everything else is just outright replaced.
 */
-fn merge_manifest(mut into_t: toml::Table, from_t: toml::Table) -> Result<toml::Table> {
+fn merge_manifest(
+    mut into_t: toml::value::Table,
+    from_t: toml::value::Table,
+) -> Result<toml::value::Table> {
     for (k, v) in from_t {
         match v {
             toml::Value::Table(from_t) => {
-                use std::collections::btree_map::Entry::*;
-
                 // Merge.
                 match into_t.entry(k) {
-                    Vacant(e) => {
+                    toml::map::Entry::Vacant(e) => {
                         e.insert(toml::Value::Table(from_t));
-                    },
-                    Occupied(e) => {
-                        let into_t = as_table_mut(e.into_mut())
-                            .ok_or((Blame::Human, "cannot merge manifests: cannot merge \
-                                table and non-table values"))?;
+                    }
+                    toml::map::Entry::Occupied(e) => {
+                        let into_t = as_table_mut(e.into_mut()).ok_or((
+                            Blame::Human,
+                            "cannot merge manifests: cannot merge \
+                                table and non-table values",
+                        ))?;
                         into_t.extend(from_t);
                     }
                 }
-            },
+            }
             v => {
                 // Just replace.
                 into_t.insert(k, v);
-            },
+            }
         }
     }
 
     return Ok(into_t);
 
-    fn as_table_mut(t: &mut toml::Value) -> Option<&mut toml::Table> {
+    fn as_table_mut(t: &mut toml::Value) -> Option<&mut toml::value::Table> {
         match *t {
             toml::Value::Table(ref mut t) => Some(t),
-            _ => None
+            _ => None,
         }
     }
 }
@@ -1052,7 +1114,7 @@ fn merge_manifest(mut into_t: toml::Table, from_t: toml::Table) -> Result<toml::
 /**
 Given a Cargo manifest, attempts to rewrite relative file paths to absolute ones, allowing the manifest to be relocated.
 */
-fn fix_manifest_paths(mani: toml::Table, base: &Path) -> Result<toml::Table> {
+fn fix_manifest_paths(mani: toml::value::Table, base: &Path) -> Result<toml::value::Table> {
     // Values that need to be rewritten:
     let paths: &[&[&str]] = &[
         &["build-dependencies", "*", "path"],
@@ -1066,17 +1128,13 @@ fn fix_manifest_paths(mani: toml::Table, base: &Path) -> Result<toml::Table> {
 
     for path in paths {
         iterate_toml_mut_path(&mut mani, path, &mut |v| {
-            match *v {
-                toml::Value::String(ref mut s) => {
-                    if Path::new(s).is_relative() {
-                        let p = base.join(&*s);
-                        match p.to_str() {
-                            Some(p) => *s = p.into(),
-                            None => {},
-                        }
+            if let toml::Value::String(ref mut s) = *v {
+                if Path::new(s).is_relative() {
+                    let p = base.join(&*s);
+                    if let Some(p) = p.to_str() {
+                        *s = p.into()
                     }
-                },
-                _ => {}
+                }
             }
             Ok(())
         })?
@@ -1092,8 +1150,10 @@ fn fix_manifest_paths(mani: toml::Table, base: &Path) -> Result<toml::Table> {
 Iterates over the specified TOML values via a path specification.
 */
 fn iterate_toml_mut_path<F>(base: &mut toml::Value, path: &[&str], on_each: &mut F) -> Result<()>
-where F: FnMut(&mut toml::Value) -> Result<()> {
-    if path.len() == 0 {
+where
+    F: FnMut(&mut toml::Value) -> Result<()>,
+{
+    if path.is_empty() {
         return on_each(base);
     }
 
@@ -1101,25 +1161,14 @@ where F: FnMut(&mut toml::Value) -> Result<()> {
     let tail = &path[1..];
 
     if cur == "*" {
-        match *base {
-            toml::Value::Table(ref mut tab) => {
-                for (_, v) in tab {
-                    iterate_toml_mut_path(v, tail, on_each)?;
-                }
-            },
-            _ => {},
+        if let toml::Value::Table(ref mut tab) = *base {
+            for (_, v) in tab {
+                iterate_toml_mut_path(v, tail, on_each)?;
+            }
         }
-    } else {
-        match *base {
-            toml::Value::Table(ref mut tab) => {
-                match tab.get_mut(cur) {
-                    Some(v) => {
-                        iterate_toml_mut_path(v, tail, on_each)?;
-                    },
-                    None => {},
-                }
-            },
-            _ => {},
+    } else if let toml::Value::Table(ref mut tab) = *base {
+        if let Some(v) = tab.get_mut(cur) {
+            iterate_toml_mut_path(v, tail, on_each)?;
         }
     }
 
